#define LAMBDA_UV		340.0
#define LAMBDA_BLUE		460.0
#define LAMBDA_GREEN		520.0
#define LAMBDA_RED		700.0
#define LAMBDA_IR		1000.0

static void
doppler_shift( rgb_color *color, float freq_ratio )
{
	float r0,g0,b0;
	float k;

	r0 = color->r;
	g0 = color->g;
	b0 = color->b;

	/* Convert frequency ratio into wavelength ratio */
	k = 1.0 / freq_ratio;

	/* Calculate red component */
	if (LAMBDA_RED < (k * LAMBDA_UV))
		color->r = LAMBDA_RED * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_RED < (k * LAMBDA_BLUE))
		color->r = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_RED - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_RED < (k * LAMBDA_GREEN))
		color->r = b0 + ((LAMBDA_RED - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_RED < (k * LAMBDA_RED))
		color->r = g0 + ((LAMBDA_RED - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_RED < (k * LAMBDA_IR))
		color->r = r0 + ((LAMBDA_RED - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		color->r = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_RED);

	/* Calculate green component */
	if (LAMBDA_GREEN < (k * LAMBDA_UV))
		color->g = LAMBDA_GREEN * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_GREEN < (k * LAMBDA_BLUE))
		color->g = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_GREEN - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_GREEN < (k * LAMBDA_GREEN))
		color->g = b0 + ((LAMBDA_GREEN - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_GREEN < (k * LAMBDA_RED))
		color->g = g0 + ((LAMBDA_GREEN - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_GREEN < (k * LAMBDA_IR))
		color->g = r0 + ((LAMBDA_GREEN - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		color->g = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_GREEN);

	/* Calculate blue component */
	if (LAMBDA_BLUE < (k * LAMBDA_UV))
		color->b = LAMBDA_BLUE * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_BLUE < (k * LAMBDA_BLUE))
		color->b = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_BLUE - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_BLUE < (k * LAMBDA_GREEN))
		color->b = b0 + ((LAMBDA_BLUE - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_BLUE < (k * LAMBDA_RED))
		color->b = g0 + ((LAMBDA_BLUE - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_BLUE < (k * LAMBDA_IR))
		color->b = r0 + ((LAMBDA_BLUE - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		color->b = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_BLUE);
}


/* Doppler frequency shift, for reflected light rays
 * Again, this is straight from BACKLIGHT */
static void
doppler_shift_ref( rgb_color *color, float freq_ratio )
{
	float r0,g0,b0;
	float k;

	r0 = color->r;
	g0 = color->g;
	b0 = color->b;

	/* Convert frequency ratio into wavelength ratio */
	k = 1.0 / freq_ratio;

	/* Calculate red component */
	if (LAMBDA_RED < (k * LAMBDA_BLUE))
		color->r = b0;
	else if (LAMBDA_RED < (k * LAMBDA_GREEN))
		color->r = b0 + ((LAMBDA_RED - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_RED < (k * LAMBDA_RED))
		color->r = g0 + ((LAMBDA_RED - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else
		color->r = r0;

	/* Calculate green component */
	if (LAMBDA_GREEN < (k * LAMBDA_BLUE))
		color->g = b0;
	else if (LAMBDA_GREEN < (k * LAMBDA_GREEN))
		color->g = b0 + ((LAMBDA_GREEN - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_GREEN < (k * LAMBDA_RED))
		color->g = g0 + ((LAMBDA_GREEN - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else
		color->g = r0;

	/* Calculate blue component */
	if (LAMBDA_BLUE < (k * LAMBDA_BLUE))
		color->b = b0;
	else if (LAMBDA_BLUE < (k * LAMBDA_GREEN))
		color->b = b0 + ((LAMBDA_BLUE - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_BLUE < (k * LAMBDA_RED))
		color->b = g0 + ((LAMBDA_BLUE - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else
		color->b = r0;
}