// TerrellView.cpp : implementation of the CTerrellView class
//

#include "stdafx.h"
#include "Terrell.h"

#include "TerrellDoc.h"
#include "TerrellView.h"
//Zelf toegevoegd
#include "Gl\Gl.h"
#include "Gl\Glu.h"
#include "OptiesDlg.h"
#include "AdvOptions.h"
#include "Voorwerp.h"
#include "math.h"
//Einde

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	MAX_AANT_VW	1000
#define LAMBDA_UV		340.0
#define LAMBDA_BLUE		460.0
#define LAMBDA_GREEN	520.0
#define LAMBDA_RED		700.0
#define LAMBDA_IR		1000.0

//Zelf toegevoegd
unsigned char threeto8[8] =
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] =
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] =
{
	0, 255
};

static int defaultOverride[13] =
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

static PALETTEENTRY defaultPalEntry[20] =
{
	
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};

//Hulpfunctie
void dopplershift(double *r, double *g, double *b, double freq_ratio);
void enlarge(double magn_fact);
void translate(double x, double y, double z);

//Voorwerp in ruimte
Voorwerp vw[MAX_AANT_VW];

/////////////////////////////////////////////////////////////////////////////
// CTerrellView

IMPLEMENT_DYNCREATE(CTerrellView, CView)

BEGIN_MESSAGE_MAP(CTerrellView, CView)
	//{{AFX_MSG_MAP(CTerrellView)
	ON_COMMAND(ID_FILE_PLAY, OnFilePlay)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAY, OnUpdateFilePlay)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_COMMAND(ID_FILE_OPTIES, OnFileOpties)
	ON_COMMAND(ID_FILE_REWIND, OnFileRewind)
	ON_COMMAND(ID_VIEW_ADVANCEDOPTIONS, OnViewAdvancedoptions)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTerrellView construction/destruction

CTerrellView::CTerrellView()
{
	m_play = false;

}

CTerrellView::~CTerrellView()
{
}

BOOL CTerrellView::PreCreateWindow(CREATESTRUCT& cs)
{
	//Zelf Toegevoegd
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	//Einde

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTerrellView drawing

void CTerrellView::OnDraw(CDC* pDC)
{
	CTerrellDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
	DrawScene();
}

/////////////////////////////////////////////////////////////////////////////
// CTerrellView diagnostics

#ifdef _DEBUG

void CTerrellView::AssertValid() const
{
	CView::AssertValid();
}

void CTerrellView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTerrellDoc* CTerrellView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTerrellDoc)));
	return (CTerrellDoc*)m_pDocument;
}

#endif //_DEBUG
/////////////////////////////////////////////////////////////////////////////
// CTerrellView message handlers

void CTerrellView::OnFilePlay() 
{
	m_play = m_play ? FALSE : TRUE;
	if(m_play)
		SetTimer(1, 5, NULL);
	else
		KillTimer(1);
	
}

void CTerrellView::OnUpdateFilePlay(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_play);	
}

int CTerrellView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	//Zelf gedaan
	Init();
	//Einde

	return 0;
}

void CTerrellView::OnDestroy() 
{
	HGLRC hrc;

	KillTimer(1);

	hrc = ::wglGetCurrentContext();

	::wglMakeCurrent(NULL, NULL);

	if(hrc)
		::wglDeleteContext(hrc);
	
	if(m_pOldPalette)
		m_pDC->SelectPalette(m_pOldPalette, FALSE);

	if(m_pDC)
		delete m_pDC;

	CView::OnDestroy();
}

void CTerrellView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	if(cy > 0)
	{
		glViewport(0, 0, cx, cy);

		if((m_oldRect.right > cx) || (m_oldRect.bottom > cy))
			RedrawWindow();

		m_oldRect.right = cx;
		m_oldRect.bottom = cy;

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60.0f, (GLdouble)cx/cy, 1.0f, 200.0f);
		glMatrixMode(GL_MODELVIEW);
	}
}

void CTerrellView::OnTimer(UINT nIDEvent) 
{
	DrawScene();
	
	CView::OnTimer(nIDEvent);

	MSG msg;
	while(::PeekMessage(&msg, m_hWnd, WM_TIMER, WM_TIMER, PM_REMOVE));
}

void CTerrellView::OnViewAdvancedoptions() 
{
	AdvOptions dlg;
	BOOL oude_kleur;
	int oude_scene;
	int vwnr;
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	dlg.m_blok_kleur = pDoc->use_blok_kleur;
	dlg.m_use_cockpit = pDoc->use_cockpit;
	oude_kleur = pDoc->use_blok_kleur;
	oude_scene = pDoc->scene_number;

	int result = dlg.DoModal();
	if(result == IDOK)
	{
		pDoc->use_blok_kleur = dlg.m_blok_kleur;
		pDoc->use_cockpit = dlg.m_use_cockpit;
		pDoc->scene_number = dlg.scene;
	}
	if(oude_kleur != pDoc->use_blok_kleur)
	{
		vwnr = 0;
		for(double x = 0.0; x < 1.01; x += 0.11)
		{
			for(double y = 0.0; y < 1.01; y += 0.11)
			{
				for(double z = 0.0; z < 1.01; z += 0.11)
				{
					if(pDoc->use_blok_kleur)
					{			
						vw[vwnr].set_color(x, y, z);
					}
					else
					{
						vw[vwnr].set_color(0.5, 0.5, 0.5);
					}
					vwnr++;
				}
			}
		}
	}
	if(oude_scene != pDoc->scene_number)
	{
		VoorwerpInit(pDoc->scene_number);
	}
}

void CTerrellView::OnFileOpties() 
{
	int oude_richting, vwnr;
	double oude_snelheid;
	OptiesDlg dlg;
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	dlg.m_doppler		= pDoc->use_doppler;
	dlg.m_intensiteit	= pDoc->use_intensiteit;
	dlg.m_lorentz		= pDoc->use_lorentz;
	dlg.m_terrell		= pDoc->use_terrell;
	dlg.m_richting		= pDoc->bewegings_richting;
	dlg.m_snelheid		= pDoc->snelheid;
	dlg.m_wortel_int	= pDoc->use_wortel_int;
	oude_richting = pDoc->bewegings_richting;
	oude_snelheid = pDoc->snelheid;
	int result = dlg.DoModal();
	if(result == IDOK)
	{
		pDoc->use_doppler		= dlg.m_doppler;
		pDoc->use_intensiteit	= dlg.m_intensiteit;
		pDoc->use_lorentz		= dlg.m_lorentz;
		pDoc->use_terrell		= dlg.m_terrell;
		pDoc->bewegings_richting = dlg.m_richting;
		pDoc->snelheid			= dlg.m_snelheid;
		pDoc->use_wortel_int	= dlg.m_wortel_int;
	}
	for(vwnr = 0; vw[vwnr].gebruikt; vwnr++)
	{
		if((oude_richting != pDoc->bewegings_richting) || (oude_snelheid != pDoc->snelheid))
		{
			pDoc->bewegings_richting	= oude_richting;
			pDoc->snelheid				= oude_snelheid;
			vw[vwnr].tijd_naar_nul();
			pDoc->bewegings_richting	= dlg.m_richting;
			pDoc->snelheid				= dlg.m_snelheid;
		}
		switch(pDoc->bewegings_richting)
		{
		case 0:
			vw[vwnr].set_velocity(pDoc->snelheid, 0.0, 0.0);
			break;
		case 1:
			vw[vwnr].set_velocity(0.0, pDoc->snelheid, 0.0);
			break;
		case 2:
			vw[vwnr].set_velocity(0.0, 0.0, pDoc->snelheid);
			break;
		}
		vw[vwnr].calculate_lorentz();
	}
}

void CTerrellView::OnFileRewind() 
{
	/*int vwnr;
	for(vwnr = 0; vw[vwnr].gebruikt; vwnr++)
	{
		vw[vwnr].tijd_naar_nul();
	}*/
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	VoorwerpInit(pDoc->scene_number);
}
/////////////////////////////////////////////////////////////////////////////
//OpenGL functies

void CTerrellView::Init()
{
	PIXELFORMATDESCRIPTOR	pfd;
	HGLRC	hrc;
	int n;
	GLfloat	fMaxObjSize, fAspect;
	GLfloat	fNearPlane, fFarPlane;
	//Zelf toegevoegd
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->use_doppler		= false;
	pDoc->use_intensiteit	= false;
	pDoc->use_lorentz		= false;
	pDoc->use_terrell		= false;
	pDoc->use_blok_kleur	= true;
	pDoc->use_cockpit		= false;
	pDoc->use_wortel_int	= false;
	pDoc->bewegings_richting = 0;
	pDoc->snelheid			= 0.8;
	pDoc->scene_number		= 1;
	pDoc->use_wortel_int	= false;
	//Einde

	m_pDC = new CClientDC(this);

	ASSERT(m_pDC != NULL);

	if(!bSetupPixelFormat())
		return;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	CreateRGBPalette();

	hrc = wglCreateContext(m_pDC->GetSafeHdc());
	wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

	GetClientRect(&m_oldRect);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);

	if(m_oldRect.bottom)
		fAspect = (GLfloat)m_oldRect.right/m_oldRect.bottom;
	else
		fAspect = 1.0f;
	fNearPlane = 1.0f;
	fFarPlane = 200.0f;
	fMaxObjSize = 3.0f;
	m_fRadius = fNearPlane + fMaxObjSize / 2.0f;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0f, fAspect, fNearPlane, fFarPlane);
	glMatrixMode(GL_MODELVIEW);
	//Voorwerpen initialiseren
	VoorwerpInit(pDoc->scene_number);
}

void CTerrellView::VoorwerpInit(int scene)
{
	int		vwnr, j, i, gebouwteller, wegteller;
	double	xas, yas, zas, w, e, q;
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	for(vwnr = 0; vwnr < MAX_AANT_VW; vwnr++)
	{
		vw[vwnr].disable();
	}
	switch(scene)
	{
	case 0:
		pDoc->use_cockpit = false;
		vwnr = 0;
		for(xas = -4.5; xas < 4.6; xas += 1.0)
		{
			for(yas = -4.5; yas < 4.6; yas += 1.0)
			{
				for(zas = -1.1; zas > -11.1; zas -= 1.0)
				{
					vw[vwnr].enable();
					switch(pDoc->bewegings_richting)
					{
					case 0:
						vw[vwnr].set_velocity(pDoc->snelheid, 0.0, 0.0);
						break;
					case 1:
						vw[vwnr].set_velocity(0.0, pDoc->snelheid, 0.0);
						break;
					case 2:
						vw[vwnr].set_velocity(0.0, 0.0, pDoc->snelheid);
						break;
					}
					vw[vwnr].make_cube(0.4);
					if(pDoc->use_blok_kleur)
					{
						vw[vwnr].set_color((xas + 4.5)/9.1, (yas + 4.5)/9.1, (zas - 1.1)/-9.1);
					}
					else
					{
						vw[vwnr].set_color(0.5f, 0.5f, 0.5f);
					}
					vw[vwnr].set_location(xas, yas, zas);
					vw[vwnr].calculate_lorentz();
					vwnr++;
				}
			}
		}
		break;
	case 1:
		pDoc->use_cockpit = true;
		pDoc->bewegings_richting = 0;
		vwnr = 0;
		for(j = 0; j < 96; j++)
		{
			vw[j].enable();
			vw[j].tijd = 0.0;
		}
//achterwiel
		vw[0].make_wheel(0.08);
		vw[0].set_location(-0.18, -0.25,  -2.0);
		vw[0].set_color(1.0f, 0.5f, 0.5f);
//voorwiel
		vw[1].make_wheel(0.08);
		vw[1].set_location( 0.18, -0.25,  -2.0);
		vw[1].set_color(1.0f, 0.5f, 0.5f);
//framestang
		vw[2].make_balk(0.36, 0.03, 0.03);
		vw[2].set_location(-0.02, -0.16, -2.0);
		vw[2].set_color(0.5, 1.0, 1.0);
//lijf
		vw[3].make_balk(0.06, 0.15, 0.12);
		vw[3].rotate_z(10.0);
		vw[3].set_location(0.0, -0.07, -2.0);
		vw[3].set_color(1.0, 0.0, 0.0);
//hoofd
		vw[4].make_cube(0.06);
		vw[4].set_location(0.0, 0.05, -2.0);
		vw[4].set_color(0.8, 0.3, 0.4);
//Linkeroog
		vw[16].make_balk(0.001, 0.025, 0.025);
		vw[16].set_location(0.03, 0.06, -2.015);
		vw[16].set_color(0.0, 0.0, 0.1);
//Rechteroog
		vw[17].make_balk(0.001, 0.025, 0.025);
		vw[17].set_location(0.03, 0.06, -1.985);
		vw[17].set_color(0.0, 0.0, 0.1);
//Petje
		vw[5].make_balk(0.062, 0.02, 0.062);
		vw[5].set_location(0.0, 0.091, -2.0);
		vw[5].set_color(0.5, 1.0, 0.5);
//stuur
		vw[6].make_balk(0.03, 0.03, 0.14);
		vw[6].set_location(0.12, -0.08, -2.0);
		vw[6].set_color(0.5, 1.0, 1.0);
//stuurstang
		vw[7].make_balk(0.20, 0.03, 0.03);
		vw[7].rotate_z(70.0);
		vw[7].set_location(0.15, -0.17, -2.0);
		vw[7].set_color(0.5, 1.0, 1.0);
//linkerarm
		vw[8].make_balk(0.04, 0.14, 0.04);
		vw[8].rotate_z(-60.0);
		vw[8].set_location(0.04, -0.05, -2.09);
		vw[8].set_color(1.0, 0.0, 0.0);
//rechterarm
		vw[9].make_balk(0.04, 0.14, 0.04);
		vw[9].rotate_z(-60.0);
		vw[9].set_location(0.04, -0.05, -1.91);
		vw[9].set_color(1.0, 0.0, 0.0);
//linkerbovenbeen
		vw[10].make_balk(0.04, 0.14, 0.04);
		vw[10].rotate_z(-50.0);
		vw[10].set_location(0.04, -0.15, -1.96);
		vw[10].set_color(0.0, 1.0, 0.0);
//rechterbovenbeen
		vw[11].make_balk(0.04, 0.14, 0.04);
		vw[11].rotate_z(-50.0);
		vw[11].set_location(0.04, -0.15, -2.04);
		vw[11].set_color(0.0, 1.0, 0.0);
//linkeronderbeen
		vw[12].make_balk(0.04, 0.14, 0.04);
		vw[12].rotate_z(55.0);
		vw[12].set_location(0.04, -0.21, -1.96);
		vw[12].set_color(0.0, 1.0, 0.0);
//rechteronderbeen
		vw[13].make_balk(0.04, 0.14, 0.04);
		vw[13].rotate_z(55.0);
		vw[13].set_location(0.04, -0.21, -2.04);
		vw[13].set_color(0.0, 1.0, 0.0);
//linkervoet
		vw[14].make_balk(0.08, 0.04, 0.041);
		vw[14].rotate_z(30.0);
		vw[14].set_location(0.02, -0.26, -1.96);
		vw[14].set_color(0.1, 0.08, 0.2);
//rechtervoet
		vw[15].make_balk(0.08, 0.04, 0.041);
		vw[15].rotate_z(30.0);
		vw[15].set_location(0.02, -0.26, -2.04);
		vw[15].set_color(0.1, 0.08, 0.2);
//Fietser groter maken en verplaatsen
		enlarge(2.0);
		translate(-2.0, 0.33, 2.0);
//wegelementen
		wegteller = 18;
		for(q = -10.0; q < 10.01; q += 0.5)
		{
			vw[wegteller].make_balk(0.5, 0.02, 5.0);
			vw[wegteller].set_location(q, -0.34, -3.6);
			vw[wegteller].set_color(0.1, 0.1, 0.1);
			wegteller++;
		}
//Gebouwen
		gebouwteller = 59;
		for(w = -16.0; w < 16.1; w += 4.0)
		{
			for(e = 0.17; e < 3.18; e += 1.0)
			{
				vw[gebouwteller].make_balk(3.0, 1.0, 2.5);
				vw[gebouwteller].set_location(w, e, -4.0);
				vw[gebouwteller].set_color(0.4 + 0.2 * cos(w), 0.4 + 0.2 * cos(w), 0.45 + 0.2 * cos(w));
				gebouwteller++;
			}
		}
		for(i = 0; i < 96; i++)
		{
			switch(pDoc->bewegings_richting)
			{
			case 0:
				vw[i].set_velocity(pDoc->snelheid, 0.0, 0.0);
				break;
			case 1:
				vw[i].set_velocity(0.0, pDoc->snelheid, 0.0);
				break;
			case 2:
				vw[i].set_velocity(0.0, 0.0, pDoc->snelheid);
				break;
			}
			vw[i].calculate_lorentz();
			vwnr++;
		}
		break;
//Weg met paaltjes
	case 2:
		pDoc->use_cockpit = false;
		pDoc->bewegings_richting = 2;
		vwnr = 0;
		for(xas = -6.0; xas < 6.1; xas += 4.0)
		{
			for(yas = -0.25; yas < 4.75; yas += 1.0)
			{
				for(zas = -1.0; zas > -91.1; zas -= 3.0)
				{
					vw[vwnr].enable();
					switch(pDoc->bewegings_richting)
					{
					case 0:
						vw[vwnr].set_velocity(pDoc->snelheid, 0.0, 0.0);
						break;
					case 1:
						vw[vwnr].set_velocity(0.0, pDoc->snelheid, 0.0);
						break;
					case 2:
						vw[vwnr].set_velocity(0.0, 0.0, pDoc->snelheid);
						break;
					}
					vw[vwnr].make_cube(1.0);
					vw[vwnr].set_color(0.5 + 0.5 * sin(xas), 0.5 + 0.5 * sin(xas + zas), 0.5 + 0.5 * sin(zas));
					vw[vwnr].set_location(xas, yas, zas);
					vw[vwnr].calculate_lorentz();
					vwnr++;
				}
			}
		}
		for(xas = -5.0; xas < 5.1; xas += 2.5)
		{
			for(zas = 0.0; zas > -92.1; zas -= 2.5)
			{
				vw[vwnr].enable();
				switch(pDoc->bewegings_richting)
				{
				case 0:
					vw[vwnr].set_velocity(pDoc->snelheid, 0.0, 0.0);
					break;
				case 1:
					vw[vwnr].set_velocity(0.0, pDoc->snelheid, 0.0);
					break;
				case 2:
					vw[vwnr].set_velocity(0.0, 0.0, pDoc->snelheid);
					break;
				}
				vw[vwnr].make_balk(2.5, 0.01, 2.5);
				if((xas > -0.1) && (xas < 0.1))
				{
					vw[vwnr].set_color(0.1, 0.1, 0.1);	
				}
				else
				{
					vw[vwnr].set_color(0.5, 0.5, 0.5);
				}
				vw[vwnr].set_location(xas, -0.755, zas);
				vw[vwnr].calculate_lorentz();
				vwnr++;
			}
		}
		break;
	}
}

BOOL CTerrellView::bSetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW |
		PFD_SUPPORT_OPENGL |
		PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0,0,0,0,0,0,
		0,
		0,
		0,
		0,
		32,
		0,
		0,
		PFD_MAIN_PLANE,
		0,
		0,0,0
	};
	int pixelformat;

	if((pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0)
	{
		MessageBox("ChoosePixelFormat failed");
		return FALSE;
	}

	if(SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
	{
		MessageBox("SetPixelFormat failed");
		return FALSE;
	}

	return TRUE;
}

unsigned char CTerrellView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
	unsigned char val;
	
	val = (unsigned char) (i >> shift);
	switch(nbits)
	{
	case 1:
		val &= 0x1;
		return oneto8[val];
	case 2:
		val &= 0x3;
		return twoto8[val];
	case 3:
		val &= 0x7;
		return threeto8[val];
	default:
		return 0;
	}
}

void CTerrellView::CreateRGBPalette()
{
	PIXELFORMATDESCRIPTOR pfd;
	LOGPALETTE *pPal;
	int n, i;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	if(pfd.dwFlags & PFD_NEED_PALETTE)
	{
		n = 1 << pfd.cColorBits;
		pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

		ASSERT( pPal != NULL);

		pPal->palVersion = 0x300;
		pPal->palNumEntries = n;
		for(i = 0 ;i < n ; i++)
		{
			pPal->palPalEntry[i].peRed = ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
			pPal->palPalEntry[i].peGreen = ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
			pPal->palPalEntry[i].peBlue = ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
			pPal->palPalEntry[i].peFlags = 0;
		}

		if((pfd.cColorBits == 8)&&
			(pfd.cRedBits == 3) && (pfd.cRedShift == 0) &&
			(pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
			(pfd.cBlueBits == 2) && (pfd.cBlueShift == 6)
			)
		{
			for(i = 1; i <= 12; i++)
				pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
		}
		m_cPalette.CreatePalette(pPal);
		delete pPal;

		m_pOldPalette = m_pDC->SelectPalette(&m_cPalette, FALSE);
		m_pDC->RealizePalette();
	}
}


void CTerrellView::DrawScene(void)
{
	static BOOL bBusy = FALSE;
	double x, y, z, beta, intm;
	int pnt, vwnr;
	double gamma, gamma2;
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(bBusy)
		return;
	bBusy = TRUE;
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	for(vwnr = 0; vw[vwnr].gebruikt; vwnr++)
		vw[vwnr].move(0.05);
	
			
	glPushMatrix();
	glBegin(GL_QUADS);
	for(vwnr = 0; vw[vwnr].gebruikt; vwnr++)
	{
		for(pnt = 0; pnt < vw[vwnr].aantal_punten; pnt++)
		{
			beta = pDoc->snelheid;
			gamma = 1/sqrt(1-(beta * beta));
			gamma2 = 1/(1-(beta*beta));
			//Lorentz contractie toepassen
			if(pDoc->use_lorentz)
			{
				x = vw[vwnr].lorentz_punt[pnt][0];
				y = vw[vwnr].lorentz_punt[pnt][1];
				z = vw[vwnr].lorentz_punt[pnt][2];
			}
			else
			{
				x = vw[vwnr].punt[pnt][0];
				y = vw[vwnr].punt[pnt][1];
				z = vw[vwnr].punt[pnt][2];
			}
			//Terrell effect
			if(pDoc->use_terrell)
			{
				x = vw[vwnr].punt[pnt][0];
				y = vw[vwnr].punt[pnt][1];
				z = vw[vwnr].punt[pnt][2];
				switch(pDoc->bewegings_richting)
				{
				case 0:
					x = gamma * (x - (beta * sqrt((x * x) + (y * y) + (z * z)))); 
					break;
				case 1:
					y = gamma * (y - (beta * sqrt((x * x) + (y * y) + (z * z))));
					break;
				case 2: 
					z = gamma * (z - (beta * sqrt((x * x) + (y * y) + (z * z))));
					break;
				}
			}
			//Intensiteits modulatie
			if(pDoc->use_intensiteit)
			{
				switch(pDoc->bewegings_richting)
				{
				case 0: intm = (1+((beta * x)/sqrt((x*x)+(y*y)+(z*z))))*(1+((beta * x)/sqrt((x*x)+(y*y)+(z*z))))*gamma2; break;
				case 1: intm = (1+((beta * y)/sqrt((x*x)+(y*y)+(z*z))))*(1+((beta * y)/sqrt((x*x)+(y*y)+(z*z))))*gamma2; break;
				case 2: intm = (1+((beta * z)/sqrt((x*x)+(y*y)+(z*z))))*(1+((beta * z)/sqrt((x*x)+(y*y)+(z*z))))*gamma2; break;
				}
			}
			else
			{
				intm = 1.0;
			}
			if(pDoc->use_wortel_int)
				intm = sqrt(intm);
			if(pDoc->use_doppler)
			{
				double freqratio;
				double kleur[3];
				switch(pDoc->bewegings_richting)
				{
				case 0: freqratio = (1-((beta * x)/sqrt((x*x)+(y*y)+(z*z))))*gamma; break;
				case 1: freqratio = (1-((beta * y)/sqrt((x*x)+(y*y)+(z*z))))*gamma; break;
				case 2: freqratio = (1-((beta * z)/sqrt((x*x)+(y*y)+(z*z))))*gamma; break;
				}
				kleur[0] = vw[vwnr].kleur[0];
				kleur[1] = vw[vwnr].kleur[1];
				kleur[2] = vw[vwnr].kleur[2];
				dopplershift(&kleur[0], &kleur[1], &kleur[2], freqratio);
				glColor3d(kleur[0] / intm, kleur[1] / intm, kleur[2] / intm);
			}
			else
			{
				glColor3d(vw[vwnr].kleur[0] / intm, vw[vwnr].kleur[1] / intm, vw[vwnr].kleur[2] / intm);
			}
			glVertex3d(x, y, z);
		}
	}
	glEnd();
	glPopMatrix();
	if(pDoc->use_cockpit)
	{
		DrawCockpit();
	}
	glFinish();
	SwapBuffers(wglGetCurrentDC());
	bBusy = FALSE;
}

void CTerrellView::DrawCockpit()
{
	double v = (vw[0].snelheid[0] + vw[0].snelheid[1] + vw[0].snelheid[2])/2.5;
	CTerrellDoc *pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	glPushMatrix();
	glBegin(GL_QUADS);
		glColor3d(0.6, 0.6, 0.6);
		glVertex3d(-3.0, -0.25, -1.001);
		glColor3d(0.6, 0.6, 0.6);
		glVertex3d( 3.0, -0.25, -1.001);
		glColor3d(0.4, 0.4, 0.4);
		glVertex3d( 3.0, -0.75, -1.001);
		glColor3d(0.4, 0.4, 0.4);
		glVertex3d(-3.0, -0.75, -1.001);
	glEnd();
	glBegin(GL_QUADS);
		glColor3d(0.0, 0.9, 0.0);
		glVertex3d(v, -0.3, -1.0);
		glVertex3d(0.0, -0.3, -1.0);
		glVertex3d(0.0, -0.35, -1.0);
		glVertex3d(v, -0.35, -1.0);
	glEnd();
	glBegin(GL_QUADS);
		glColor3d(0.0, 0.0, 0.0);
		glVertex3d(-0.4, -0.29, -1.0005);
		glVertex3d(0.4, -0.29, -1.0005);
		glVertex3d(0.4, -0.36, -1.0005);
		glVertex3d(-0.4, -0.36, -1.0005);
	glEnd();
	glPopMatrix();
}

//EXTRA HULPFUNCTIES
void enlarge(double magn_fact)
{
	for(int i = 0; vw[i].gebruikt; i++)
	{
		for(int k = 0; k < vw[i].aantal_punten; k++)
		{
			for(int j = 0; j < 3; j++)
			{
				vw[i].punt[k][j] *= magn_fact;
				vw[i].lorentz_punt[k][j] *= magn_fact;
			}
		}
	}
}

void translate(double x, double y, double z)
{
	for(int i = 0; vw[i].gebruikt; i++)
	{
		for(int k = 0; k < vw[i].aantal_punten; k++)
		{
			vw[i].punt[k][0] += x;
			vw[i].punt[k][1] += y;
			vw[i].punt[k][2] += z;
			vw[i].lorentz_punt[k][0] += x;
			vw[i].lorentz_punt[k][1] += y;
			vw[i].lorentz_punt[k][2] += z;

		}
	}
}

void dopplershift(double *r, double *g, double *b, double freq_ratio)
{
	double r0,g0,b0;
	double k;

	r0 = *r;
	g0 = *g;
	b0 = *b;

	/* Convert frequency ratio into wavelength ratio */
	k = 1.0 / freq_ratio;

	/* Calculate red component */
	if (LAMBDA_RED < (k * LAMBDA_UV))
		*r = LAMBDA_RED * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_RED < (k * LAMBDA_BLUE))
		*r = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_RED - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_RED < (k * LAMBDA_GREEN))
		*r = b0 + ((LAMBDA_RED - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_RED < (k * LAMBDA_RED))
		*r = g0 + ((LAMBDA_RED - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_RED < (k * LAMBDA_IR))
		*r = r0 + ((LAMBDA_RED - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		*r = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_RED);

	/* Calculate green component */
	if (LAMBDA_GREEN < (k * LAMBDA_UV))
		*g = LAMBDA_GREEN * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_GREEN < (k * LAMBDA_BLUE))
		*g = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_GREEN - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_GREEN < (k * LAMBDA_GREEN))
		*g = b0 + ((LAMBDA_GREEN - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_GREEN < (k * LAMBDA_RED))
		*g = g0 + ((LAMBDA_GREEN - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_GREEN < (k * LAMBDA_IR))
		*g = r0 + ((LAMBDA_GREEN - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		*g = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_GREEN);

	/* Calculate blue component */
	if (LAMBDA_BLUE < (k * LAMBDA_UV))
		*b = LAMBDA_BLUE * ((0.5 * b0) + (0.25 * g0) + (0.125 * r0))/ (k * LAMBDA_UV);
	else if (LAMBDA_BLUE < (k * LAMBDA_BLUE))
		*b = ((0.5 * b0) + (0.25 * g0) + (0.125 * r0)) + ((LAMBDA_BLUE - (k * LAMBDA_UV)) / ((k * LAMBDA_BLUE) - (k * LAMBDA_UV))) * ((0.5 * b0) - (0.25 * g0) - (0.125 * r0));
	else if (LAMBDA_BLUE < (k * LAMBDA_GREEN))
		*b = b0 + ((LAMBDA_BLUE - (k * LAMBDA_BLUE)) / ((k * LAMBDA_GREEN) - (k * LAMBDA_BLUE))) * (- b0 + g0);
	else if (LAMBDA_BLUE < (k * LAMBDA_RED))
		*b = g0 + ((LAMBDA_BLUE - (k * LAMBDA_GREEN)) / ((k * LAMBDA_RED) - (k * LAMBDA_GREEN))) * (- g0 + r0);
	else if (LAMBDA_BLUE < (k * LAMBDA_IR))
		*b = r0 + ((LAMBDA_BLUE - (k * LAMBDA_RED)) / ((k * LAMBDA_IR) - (k * LAMBDA_RED))) * ((0.125 * b0) + (0.25 * g0) - (0.5 * r0));
	else
		*b = ((0.125 * b0) + (0.25 * g0) + (0.5 * r0)) * ((k * LAMBDA_IR) / LAMBDA_BLUE);
}








