// Voorwerp.cpp: implementation of the Voorwerp class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Terrell.h"
#include "Voorwerp.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Voorwerp::Voorwerp()
{
	gebruikt = false;
	tijd = 0.0;
}

Voorwerp::~Voorwerp()
{

}

void Voorwerp::enable()
{
	gebruikt = true;
}

void Voorwerp::disable()
{
	gebruikt = false;
}

void Voorwerp::move(double dt)
{
	int nr, coord;
	if(gebruikt)
	{
		for(nr = 0; nr < aantal_punten; nr++)
		{
			for(coord = 0; coord < 3; coord++)
			{
				punt[nr][coord]			+= (snelheid[coord] * dt);
				lorentz_punt[nr][coord] += (snelheid[coord] * dt);
			}
		}
	}
	tijd += dt;
}

void Voorwerp::tijd_naar_nul(void)
{
	int nr, coord;
	if(gebruikt)
	{
		for(nr = 0; nr < aantal_punten; nr++)
		{
			for(coord = 0; coord < 3; coord++)
			{
				punt[nr][coord]			-= (snelheid[coord] * tijd);
				lorentz_punt[nr][coord] -= (snelheid[coord] * tijd);
			}
		}
	}
	tijd = 0.0;
}

void Voorwerp::set_velocity(double x, double y, double z)
{
	snelheid[0] = x;
	snelheid[1] = y;
	snelheid[2] = z;
}

void Voorwerp::set_color(double red, double green, double blue)
{
	kleur[0] = red;
	kleur[1] = green;
	kleur[2] = blue;
}

void Voorwerp::set_location(double x, double y, double z)
{
	int nr;
	if(gebruikt)
	{
		for(nr = 0; nr < aantal_punten; nr++)
		{
			punt[nr][0] += x;
			punt[nr][1] += y;
			punt[nr][2] += z;
			lorentz_punt[nr][0] += x;
			lorentz_punt[nr][1] += y;
			lorentz_punt[nr][2] += z;
		}
	}
}

void Voorwerp::rotate_z(double angle)
{
	int i;
	double pi = 3.1415926535;
	double x_new, y_new;
	angle = angle * 2.0 * pi / 360.0;
	for(i = 0; i < aantal_punten; i++)
	{
		x_new = punt[i][0] * cos(angle) + punt[i][1] * sin(angle);
		y_new = -punt[i][0] * sin(angle) + punt[i][1] * cos(angle);
		punt[i][0] = x_new;
		punt[i][1] = y_new;
	}
}

void Voorwerp::calculate_lorentz(void)
{
	int nr, coord;
	if(gebruikt)
	{
		for(nr = 0; nr < aantal_punten; nr++)
		{
			for(coord = 0; coord < 3; coord++)
			{
				lorentz_punt[nr][coord] = punt[nr][coord] * sqrt(1.0 - (snelheid[coord] * snelheid[coord]));
			}
		}
	}
}

void Voorwerp::make_cube(double ribbe)
{
	aantal_punten = 24;
	punt[0][0] = ribbe / 2.0;
	punt[0][1] = ribbe / 2.0;
	punt[0][2] = ribbe / 2.0;
	
	punt[1][0] = ribbe / 2.0;
	punt[1][1] = ribbe / 2.0;
	punt[1][2] = -ribbe / 2.0;
	
	punt[2][0] = ribbe / 2.0;
	punt[2][1] = -ribbe / 2.0;
	punt[2][2] = -ribbe / 2.0;
	
	punt[3][0] = ribbe / 2.0;
	punt[3][1] = -ribbe / 2.0;
	punt[3][2] = ribbe / 2.0;
	
	punt[4][0] = -ribbe / 2.0;
	punt[4][1] = ribbe / 2.0;
	punt[4][2] = ribbe / 2.0;
	
	punt[5][0] = -ribbe / 2.0;
	punt[5][1] = ribbe / 2.0;
	punt[5][2] = -ribbe / 2.0;
	
	punt[6][0] = -ribbe / 2.0;
	punt[6][1] = -ribbe / 2.0;
	punt[6][2] = -ribbe / 2.0;
	
	punt[7][0] = -ribbe / 2.0;
	punt[7][1] = -ribbe / 2.0;
	punt[7][2] = ribbe / 2.0;
	
	punt[8][0] = ribbe / 2.0;
	punt[8][1] = ribbe / 2.0;
	punt[8][2] = ribbe / 2.0;
	
	punt[9][0] = ribbe / 2.0;
	punt[9][1] = -ribbe / 2.0;
	punt[9][2] = ribbe / 2.0;
	
	punt[10][0] = -ribbe / 2.0;
	punt[10][1] = -ribbe / 2.0;
	punt[10][2] = ribbe / 2.0;
	
	punt[11][0] = -ribbe / 2.0;
	punt[11][1] = ribbe / 2.0;
	punt[11][2] = ribbe / 2.0;
	
	punt[12][0] = ribbe / 2.0;
	punt[12][1] = ribbe / 2.0;
	punt[12][2] = -ribbe / 2.0;
	
	punt[13][0] = ribbe / 2.0;
	punt[13][1] = -ribbe / 2.0;
	punt[13][2] = -ribbe / 2.0;
	
	punt[14][0] = -ribbe / 2.0;
	punt[14][1] = -ribbe / 2.0;
	punt[14][2] = -ribbe / 2.0;
	
	punt[15][0] = -ribbe / 2.0;
	punt[15][1] = ribbe / 2.0;
	punt[15][2] = -ribbe / 2.0;
	
	punt[16][0] = ribbe / 2.0;
	punt[16][1] = ribbe / 2.0;
	punt[16][2] = ribbe / 2.0;
	
	punt[17][0] = ribbe / 2.0;
	punt[17][1] = ribbe / 2.0;
	punt[17][2] = -ribbe / 2.0;
	
	punt[18][0] = -ribbe / 2.0;
	punt[18][1] = ribbe / 2.0;
	punt[18][2] = -ribbe / 2.0;
	
	punt[19][0] = -ribbe / 2.0;
	punt[19][1] = ribbe / 2.0;
	punt[19][2] = ribbe / 2.0;
	
	punt[20][0] = ribbe / 2.0;
	punt[20][1] = -ribbe / 2.0;
	punt[20][2] = ribbe / 2.0;
	
	punt[21][0] = ribbe / 2.0;
	punt[21][1] = -ribbe / 2.0;
	punt[21][2] = -ribbe / 2.0;
	
	punt[22][0] = -ribbe / 2.0;
	punt[22][1] = -ribbe / 2.0;
	punt[22][2] = -ribbe / 2.0;
	
	punt[23][0] = -ribbe / 2.0;
	punt[23][1] = -ribbe / 2.0;
	punt[23][2] = ribbe / 2.0;
}

void Voorwerp::make_balk(double xribbe, double yribbe, double zribbe)
{
	aantal_punten = 24;
	punt[0][0] = xribbe / 2.0;
	punt[0][1] = yribbe / 2.0;
	punt[0][2] = zribbe / 2.0;
	
	punt[1][0] = xribbe / 2.0;
	punt[1][1] = yribbe / 2.0;
	punt[1][2] = -zribbe / 2.0;
	
	punt[2][0] = xribbe / 2.0;
	punt[2][1] = -yribbe / 2.0;
	punt[2][2] = -zribbe / 2.0;
	
	punt[3][0] = xribbe / 2.0;
	punt[3][1] = -yribbe / 2.0;
	punt[3][2] = zribbe / 2.0;
	
	punt[4][0] = -xribbe / 2.0;
	punt[4][1] = yribbe / 2.0;
	punt[4][2] = zribbe / 2.0;
	
	punt[5][0] = -xribbe / 2.0;
	punt[5][1] = yribbe / 2.0;
	punt[5][2] = -zribbe / 2.0;
	
	punt[6][0] = -xribbe / 2.0;
	punt[6][1] = -yribbe / 2.0;
	punt[6][2] = -zribbe / 2.0;
	
	punt[7][0] = -xribbe / 2.0;
	punt[7][1] = -yribbe / 2.0;
	punt[7][2] = zribbe / 2.0;
	
	punt[8][0] = xribbe / 2.0;
	punt[8][1] = yribbe / 2.0;
	punt[8][2] = zribbe / 2.0;
	
	punt[9][0] = xribbe / 2.0;
	punt[9][1] = -yribbe / 2.0;
	punt[9][2] = zribbe / 2.0;
	
	punt[10][0] = -xribbe / 2.0;
	punt[10][1] = -yribbe / 2.0;
	punt[10][2] = zribbe / 2.0;
	
	punt[11][0] = -xribbe / 2.0;
	punt[11][1] = yribbe / 2.0;
	punt[11][2] = zribbe / 2.0;
	
	punt[12][0] = xribbe / 2.0;
	punt[12][1] = yribbe / 2.0;
	punt[12][2] = -zribbe / 2.0;
	
	punt[13][0] = xribbe / 2.0;
	punt[13][1] = -yribbe / 2.0;
	punt[13][2] = -zribbe / 2.0;
	
	punt[14][0] = -xribbe / 2.0;
	punt[14][1] = -yribbe / 2.0;
	punt[14][2] = -zribbe / 2.0;
	
	punt[15][0] = -xribbe / 2.0;
	punt[15][1] = yribbe / 2.0;
	punt[15][2] = -zribbe / 2.0;
	
	punt[16][0] = xribbe / 2.0;
	punt[16][1] = yribbe / 2.0;
	punt[16][2] = zribbe / 2.0;
	
	punt[17][0] = xribbe / 2.0;
	punt[17][1] = yribbe / 2.0;
	punt[17][2] = -zribbe / 2.0;
	
	punt[18][0] = -xribbe / 2.0;
	punt[18][1] = yribbe / 2.0;
	punt[18][2] = -zribbe / 2.0;
	
	punt[19][0] = -xribbe / 2.0;
	punt[19][1] = yribbe / 2.0;
	punt[19][2] = zribbe / 2.0;
	
	punt[20][0] = xribbe / 2.0;
	punt[20][1] = -yribbe / 2.0;
	punt[20][2] = zribbe / 2.0;
	
	punt[21][0] = xribbe / 2.0;
	punt[21][1] = -yribbe / 2.0;
	punt[21][2] = -zribbe / 2.0;
	
	punt[22][0] = -xribbe / 2.0;
	punt[22][1] = -yribbe / 2.0;
	punt[22][2] = -zribbe / 2.0;
	
	punt[23][0] = -xribbe / 2.0;
	punt[23][1] = -yribbe / 2.0;
	punt[23][2] = zribbe / 2.0;
}


void Voorwerp::make_wheel(double straal)
{
	aantal_punten = 12;
	punt[0][0] = -1.0  * straal;
	punt[0][1] = -0.35 * straal;
	punt[0][2] =  0.0;

	punt[1][0] = -1.0  * straal;
	punt[1][1] =  0.35 * straal;
	punt[1][2] =  0.0;
	
	punt[2][0] = -0.35 * straal;
	punt[2][1] =  1.0  * straal;
	punt[2][2] =  0.0;
	
	punt[3][0] =  0.35 * straal;
	punt[3][1] =  1.0  * straal;
	punt[3][2] =  0.0;
	
	punt[4][0] = -1.0  * straal;
	punt[4][1] = -0.35 * straal;
	punt[4][2] =  0.0;
	
	punt[5][0] =  0.35 * straal;
	punt[5][1] =  1.0  * straal;
	punt[5][2] =  0.0;
	
	punt[6][0] =  1.0  * straal;
	punt[6][1] =  0.35 * straal;
	punt[6][2] =  0.0;
	
	punt[7][0] = -0.35 * straal;
	punt[7][1] = -1.0  * straal;
	punt[7][2] =  0.0;
	
	punt[8][0] = -0.35 * straal;
	punt[8][1] = -1.0  * straal;
	punt[8][2] =  0.0;
	
	punt[9][0] =  1.0  * straal;
	punt[9][1] =  0.35 * straal;
	punt[9][2] =  0.0;
	
	punt[10][0] =  1.0  * straal;
	punt[10][1] = -0.35 * straal;
	punt[10][2] =  0.0;
	
	punt[11][0] =  0.35 * straal;
	punt[11][1] = -1.0  * straal;
	punt[11][2] =  0.0;
}
	